﻿Imports System
Imports System.ComponentModel
Imports System.Drawing
Imports Wisej.Web

Partial Public Class Page1

    Private Const m_UnitName = "Page1"

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Sub Page1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
        Try
            With DataRepeater1
                .VirtualMode = True

                'See https://www.modernizing-applications.de/developer-blog/wisej-datarepeater-control/
                'For a smoother scrolling you can use the PrefetchItems property. 
                'The default value for PrefetchItems is 0
                'Any value greater than 0 represents the number of additional containers
                'which will be created on client side for pre-loading the next/previous items.
                .PrefetchItems = 5

                'n/a here
                .AllowDrag = False
                .AllowDrop = False

                If .VirtualMode = True Then
                    Dim data = New BindingList(Of Model)()

                    For i = 0 To 100 - 1
                        data.Add(New Model())
                    Next

                    .DataSource = data
                Else
                    Dim data = New BindingList(Of Model)()

                    For i = 0 To 100 - 1
                        data.Add(New Model() With {
                                .ID = i,
                                .Name = $"Item #{i}"
                            })
                    Next

                    .DataSource = data
                End If
            End With

            m_Resize()

        Catch ex As Exception
            ErrMsg(System.Reflection.MethodBase.GetCurrentMethod.Name(), m_UnitName)
        End Try
    End Sub

    Private Sub DataRepeater1_ItemSelected(ByVal sender As Object, ByVal e As DataRepeaterItemEventArgs) Handles DataRepeater1.ItemSelected
        button1.Enabled = e.DataRepeaterItem.ItemIndex > 0
        button2.Enabled = e.DataRepeaterItem.ItemIndex < DataRepeater1.ItemCount - 1
    End Sub

    Private Sub DataRepeater1_DragStart(ByVal sender As Object, ByVal e As EventArgs) Handles DataRepeater1.DragStart
        Dim item = DataRepeater1.CurrentItem
        Dim data = CType(DataRepeater1.DataSource, BindingList(Of Model))
        DataRepeater1.DoDragDrop(data(item.ItemIndex), DragDropEffects.Move)
    End Sub

    Private Sub DataRepeater1_DragEnter(ByVal sender As Object, ByVal e As DragEventArgs) Handles DataRepeater1.DragEnter
        If e.Data.GetDataPresent(GetType(Model)) Then
            e.Effect = DragDropEffects.Move
            e.ImageSource = "data-item.png"
            e.ImageSize = New Size(32, 32)
        End If
    End Sub

    Private Sub DataRepeater1_DragDrop(ByVal sender As Object, ByVal e As DragEventArgs) Handles DataRepeater1.DragDrop
        Dim data = CType(DataRepeater1.DataSource, BindingList(Of Model))
        Dim item = CType(e.Data.GetData(GetType(Model)), Model)
        Dim target = TryCast(e.DropTarget, DataRepeaterItem)

        If target IsNot Nothing Then
            MoveDataItem(data.IndexOf(item), target.ItemIndex)
        Else
            MoveDataItem(data.IndexOf(item), data.Count - 1)
        End If
    End Sub

    Private Sub button1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button1.Click
        Dim index = DataRepeater1.CurrentItemIndex
        MoveDataItem(index, index - 1)
    End Sub

    Private Sub button2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button2.Click
        Dim index = DataRepeater1.CurrentItemIndex
        MoveDataItem(index, index + 1)
    End Sub

    Private Sub MoveDataItem(ByVal index As Integer, ByVal destination As Integer)
        If index < 0 OrElse index = destination Then Return
        Dim data = CType(DataRepeater1.DataSource, BindingList(Of Model))
        Dim item = data(index)
        data.RemoveAt(index)
        data.Insert(destination, item)
        DataRepeater1.CurrentItemIndex = destination
    End Sub

    Private Sub DataRepeater1_ItemValueNeeded(sender As Object, e As DataRepeaterItemValueEventArgs) Handles DataRepeater1.ItemValueNeeded
        Try
            '------------------------------------------------------
            'NOTE: MUST check for ctlTest before checking for TextBox 
            If TypeOf e.Control Is ctlTest Then

                'NEVER GET HERE IF ctlTest Inherits Wisej.Web.UserControl
                'OK IF ctlTest Inherits Wisej.Web.TextBox
                Dim c As ctlTest = e.Control

                With c
                    .Ready(e.ItemIndex)
                End With

            ElseIf TypeOf e.Control Is Label Then

                e.Value = "LABEL " & e.ItemIndex

            ElseIf TypeOf e.Control Is TextBox Then

                e.Value = "TEXT " & e.ItemIndex
            End If

            '------------------------------------------------------
            'Now identify & resize this DataRepeaterItem
            For Each o As Control In DataRepeater1.Controls
                If TypeOf o Is DataRepeaterItem Then

                    Dim c As DataRepeaterItem = o

                    With c
                        If .ItemIndex = e.ItemIndex Then
                            m_Resize(o)
                        End If
                    End With
                End If
            Next

        Catch ex As Exception
            ErrMsg(System.Reflection.MethodBase.GetCurrentMethod.Name(), m_UnitName)
        End Try
    End Sub

    Private Sub Page1_Resize(sender As Object, e As EventArgs) Handles Me.Resize
        m_Resize()
    End Sub

    Private Sub m_Resize()
        Try
            '------------------------------------------------------
            'Resize the buttons
            With button1
                .Top = 20
                .Left = Me.Width - .Width - 20
            End With

            With button2
                .Top = button1.Bottom + 20
                .Left = button1.Left
            End With

            '------------------------------------------------------
            'Resize the DataRepeater
            With DataRepeater1
                .Top = 20
                .Left = 20

                .Height = Me.Height - 20
                .Width = button1.Left - 20 - .Left
            End With

            '------------------------------------------------------
            'Resize the DataRepeaterItems
            For Each c As Control In DataRepeater1.Controls
                If TypeOf c Is DataRepeaterItem Then
                    m_Resize(c)
                End If
            Next

        Catch ex As Exception
            ErrMsg(System.Reflection.MethodBase.GetCurrentMethod.Name(), m_UnitName)
        End Try
    End Sub

    Private Sub m_Resize(item As DataRepeaterItem)
        Try
            Dim cLabel As Label = Nothing
            Dim cTextBox As TextBox = Nothing
            Dim cTest As ctlTest = Nothing

            With item
                For Each c As Control In item.Controls
                    If TypeOf c Is ctlTest Then
                        cTest = c

                    ElseIf TypeOf c Is Label Then
                        cLabel = c

                    ElseIf TypeOf c Is TextBox Then
                        cTextBox = c
                    Else
                        Debug.Print("HELLO")
                    End If
                Next

                If cLabel Is Nothing Then
                    Exit Sub
                End If

                If cTextBox Is Nothing Then
                    Exit Sub
                End If

                If cTest Is Nothing Then
                    Exit Sub
                End If

                Dim d As Integer = 4

                With cLabel
                    .Left = d
                    .Top = d
                End With

                With cTextBox
                    .Left = cLabel.Left
                    .Top = cLabel.Bottom + d
                    .Width = DataRepeater1.Width - .Left - d - 40
                End With

                With cTest
                    .Left = cLabel.Left
                    .Top = cTextBox.Bottom + d

                    .Width = DataRepeater1.Width - .Left - d - 40
                    .m_Resize()   'Note this sets the .Height
                End With

                'Now we can set the item height
                .Height = cTest.Bottom + 4
            End With

        Catch ex As Exception
            ErrMsg(System.Reflection.MethodBase.GetCurrentMethod.Name(), m_UnitName)
        End Try
    End Sub

    Private Sub DataRepeater1_ItemsAdded(sender As Object, e As DataRepeaterAddRemoveItemsEventArgs) Handles DataRepeater1.ItemsAdded

        'NEVER GET HERE !!!

    End Sub

    Private Sub DataRepeater1_Scroll(sender As Object, e As ScrollEventArgs) Handles DataRepeater1.Scroll

        'NEVER GET HERE !!!

    End Sub

    Private Sub DataRepeater1_Resize(sender As Object, e As EventArgs) Handles DataRepeater1.Resize

    End Sub
End Class
